part of masamune.flutter;

/// Displays the signin dialog.
/// Specify[providers]and select the authentication type to be displayed
class UIAuth {  
  /// Displays the signin dialog.
  /// Specify [providers] and select the authentication type to be displayed
  /// 
  /// [title]: Dialog title
  /// [providers]: Provider type to authenticate
  /// [actionAfterSignIn]: 
  static Future show(
    BuildContext context,
    {
      String title,
      List<String> providers = const["google", "twitter", "facebook", "apple", "anonymous"],
      Action actionAfterSignIn,
      String twitterAPIKey,
      String twitterAPISecret,
      String dialogTitleWhenAnonymousSignIn = Const.empty,
      String dialogTextWhenAnonymousSignIn = Const.empty,
      bool forceLogin = false
    }
  ) async {
    if( context == null || title == null ) return;
    if( !forceLogin && await FirestoreAuth.isLoggedIn() ){
      if( actionAfterSignIn != null ) actionAfterSignIn();
      return;
    }
    if( isNotEmpty(twitterAPIKey) && isNotEmpty(twitterAPISecret) ){
      FirestoreAuth.options(twitterAPIKey: twitterAPIKey, twitterAPISecret: twitterAPISecret);
    }
    List<SimpleDialogOption> options = ListPool.get();
    for( String provider in providers ){
      switch(provider){
        case( "google" ):
          options.add(
            SimpleDialogOption(
              onPressed: () async {
                Navigator.of(context, rootNavigator: true).pop();
                FirestoreAuth auth = await FirestoreAuth.signInToGoogle();
                if( auth.isError || auth.isAbort ){
                  UIAuth.show(context, title:title, providers:providers, actionAfterSignIn: actionAfterSignIn, twitterAPIKey:twitterAPIKey, twitterAPISecret:twitterAPISecret, forceLogin: true);
                } else {
                  if( actionAfterSignIn != null ) actionAfterSignIn();
                }
              },
              child: Text( "Google SignIn" )
            )
          );
          break;
        case( "twitter" ):
          if( isEmpty(twitterAPIKey) || isEmpty(twitterAPISecret) ) continue;
          options.add(
            SimpleDialogOption(
              onPressed: () async {
                Navigator.of(context, rootNavigator: true).pop();
                FirestoreAuth auth = await FirestoreAuth.signInToTwitter();
                if( auth.isError || auth.isAbort ){
                  UIAuth.show(context, title:title, providers:providers, actionAfterSignIn: actionAfterSignIn, twitterAPIKey:twitterAPIKey, twitterAPISecret:twitterAPISecret, forceLogin: true);
                } else {
                  if( actionAfterSignIn != null ) actionAfterSignIn();
                }
              },
              child: Text( "Twitter SignIn" )
            )
          );
          break;
        case( "facebook" ):
          options.add(
            SimpleDialogOption(
              onPressed: () async {
                Navigator.of(context, rootNavigator: true).pop();
                FirestoreAuth auth = await FirestoreAuth.signInToFacebook();
                if( auth.isError || auth.isAbort ){
                  UIAuth.show(context, title:title, providers:providers, actionAfterSignIn: actionAfterSignIn, twitterAPIKey:twitterAPIKey, twitterAPISecret:twitterAPISecret, forceLogin: true);
                } else {
                  if( actionAfterSignIn != null ) actionAfterSignIn();
                }
              },
              child: Text( "Facebook SignIn" )
            )
          );
          break;
        case( "anonymous" ):
          options.add(
            SimpleDialogOption(
              onPressed: () async {
                Navigator.of(context, rootNavigator: true).pop();
                await FirestoreAuth.signInAnonymously();
                UIDialog.show(
                  context,
                  defaultTitle: dialogTitleWhenAnonymousSignIn,
                  defaultText: dialogTextWhenAnonymousSignIn,
                  defaultSubmitAction: () { if( actionAfterSignIn != null ) actionAfterSignIn(); }
                );                
              },
              child: Text( "Anonymous SignIn" )
            )
          );
          break;
      }
    }
    await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (context) {
        return SimpleDialog(
          title: Text(title),
          children: options
        );
      }
    );
  }
}